/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <mutex>
#include <vector>

namespace armarx
{
    // forward declaration
    namespace viz
    {
        class Client;
        struct Layer;
    } // namespace viz

    /**
     * @brief The ArVizDrawerBase class. Use this class to draw arbitrary objects.
     *
     * Instead of implementing arviz drawing in the component itself, this class helps to
     * decouple drawing from "component / processing logic".
     *
     */
    class ArVizDrawerBase
    {
      public:
        ArVizDrawerBase(armarx::viz::Client &arviz);
        virtual ~ArVizDrawerBase();

      protected:
        viz::Client &arviz();
        const viz::Client &arviz() const;

        void commit(const viz::Layer &layer);
        void commit(const std::vector<viz::Layer> &layers);

      private:
        viz::Client &arvizClient;

        std::mutex layerMtx;
    };

} // namespace armarx
