#include "CoreSegment.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "TypeIO.h"


namespace armarx::armem::d_ltm
{

    std::filesystem::path CoreSegment::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path CoreSegment::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName;
    }

    wm::CoreSegment CoreSegment::convert() const
    {
        wm::CoreSegment m;
        for (const auto& [_, s] : _container)
        {
            m.addProviderSegment(s.convert(_aronType));
        }
        return m;
    }

    void CoreSegment::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = p_ptr;

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Assuming an empty container.";
        }
        else
        {
            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    auto wms = _container.emplace(std::make_pair(k, id().withProviderSegmentName(k)));
                    wms.first->second.reload(p_ptr);
                }

                if (d.is_regular_file())
                {
                    if (auto type = TypeIO::readAronType(d.path()))
                    {
                        _aronType = type;
                    }
                }
            }
        }
    }

    void CoreSegment::append(const wm::CoreSegment& m)
    {
        std::filesystem::create_directories(_fullPath());
        TypeIO::writeAronType(_aronType, _fullPath());

        for (const auto& [k, s] : m)
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                std::filesystem::create_directory(_fullPath() / k);
                auto wms = _container.emplace(std::make_pair(k, id().withProviderSegmentName(k)));
                wms.first->second.path = path;
                wms.first->second.append(s);
            }
        }
    }
}
