#include "Entity.h"


namespace armarx::armem::ltm
{

    wm::Entity Entity::convert() const
    {
        ARMARX_INFO << "Entity: Converting with connection to: " << dbsettings.toString();

        wm::Entity m(id());
        for (const auto& [_, s] : _container)
        {
            m.addSnapshot(s.convert());
        }
        return m;
    }

    void Entity::reload()
    {
        _container.clear();

        ARMARX_INFO << "Entity: (Re)Establishing connection to: " << dbsettings.toString();

        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        mongocxx::cursor cursor = coll.find({});
        int i = 0;
        for (auto doc : cursor)
        {
            if (i > MAX_HISTORY_SIZE)
            {
                // TODO: Add logic for most queried data?
                break;
            }
            nlohmann::json json = nlohmann::json::parse(bsoncxx::to_json(doc));

            auto k = armem::Time::microSeconds(json.at("timestamp"));
            //ARMARX_INFO << "Entity: Found timestamp: " << std::to_string(k.toMicroSeconds());

            if (const auto& it = _container.find(k); it != _container.end())
            {
                throw error::ArMemError("Somehow after clearing the container a key k = " + std::to_string(k.toMicroSeconds()) + " was found. Do you have double entries in mongodb?");
            }
            else
            {
                auto wms = _container.emplace(std::make_pair(k, id().withTimestamp(k)));
                wms.first->second.dbsettings = dbsettings;
                wms.first->second.reload();
            }
            ++i;
        }
    }

    void Entity::append(const wm::Entity& m)
    {
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                // segment already exists
                // We assume that a snapshot does not change, so ignore
            }
            else
            {
                auto wms = _container.emplace(std::make_pair(k, id().withTimestamp(k)));
                wms.first->second.dbsettings = dbsettings;
                wms.first->second.setTo(s, k);
                //truncateHistoryToSize();
            }
        }
    }

    bool Entity::hasSnapshot(const Time& time) const
    {
        // check cache
        if (Base::hasSnapshot(time))
        {
            return true;
        }
        // check mongodb
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        bsoncxx::stdx::optional<bsoncxx::document::value> maybe_result = coll.find_one(document{} << "timestamp" << time.toMicroSeconds() << finalize);
        if (!maybe_result)
        {
            return false;
        }

        auto json = nlohmann::json::parse(bsoncxx::to_json(*maybe_result));
        auto id = MemoryID::fromString(json["id"].get<std::string>());
        auto instances = json["instances"];
        EntitySnapshot snapshot(id);
        snapshot.dbsettings = dbsettings;

        for (unsigned int i = 0; i < instances.size(); ++i)
        {
            EntityInstance instance(id.withInstanceIndex(i));
            snapshot.addInstance(instance);
        }

        _container.emplace(time, snapshot);
        //truncateHistoryToSize();
        return true;
    }

    std::vector<Time> Entity::getTimestamps() const
    {
        // get from cache
        std::vector<Time> ret; // = Base::getTimestamps();

        // get missing from mongodb
        mongocxx::client& client = MongoDBConnectionManager::EstablishConnection(dbsettings);
        mongocxx::database db = client[dbsettings.database];
        mongocxx::collection coll = db[id().str()];

        auto cursor = coll.find({});
        for (auto doc : cursor)
        {
            auto json = nlohmann::json::parse(bsoncxx::to_json(doc));
            auto ts = json["timestamp"].get<long>();
            ret.push_back(Time::microSeconds(ts));
        }
        return ret;
    }

    const EntitySnapshot& Entity::getSnapshot(const Time& time) const
    {
        if (!hasSnapshot(time))
        {
            throw error::MissingEntry::create<EntitySnapshotT>(toDateTimeMilliSeconds(time), *this);
        }

        // the above command already puts the reference to the cache
        return Base::getSnapshot(time);
    }

    auto Entity::getLatestItem() const -> const ContainerT::value_type&
    {
        // Directly query mongodb (cache cant know whether it is the latest or not)
        // TODO
        return Base::getLatestItem();
    }
}
