#include "MongoDBConnectionManager.h"

namespace armarx::armem::ltm
{
    bool MongoDBConnectionManager::initialized = false;
    std::map<std::string, mongocxx::client> MongoDBConnectionManager::Connections = {};


    void MongoDBConnectionManager::initialize_if()
    {
        if (!initialized)
        {
            initialized = true;
            mongocxx::instance instance{}; // This should be done only once.
        }
    }

    mongocxx::client& MongoDBConnectionManager::EstablishConnection(const MongoDBSettings& settings)
    {
        initialize_if();

        const auto uri_str = settings.uri();
        const auto& it = Connections.find(uri_str);
        if (it == Connections.end())
        {
            mongocxx::uri uri(uri_str);
            auto con = Connections.emplace(uri_str, mongocxx::client(uri));
            return con.first->second;
        }
        else
        {
            // A connection already exists. We do not need to open another one.
            return it->second;
        }
    }

    bool MongoDBConnectionManager::ConnectionIsValid(const MongoDBSettings& settings, bool force)
    {
        initialize_if();

        try
        {
            if (!force)
            {
                const auto uri_str = settings.uri();
                const auto& it = Connections.find(uri_str);
                if (it != Connections.end())
                {
                    auto admin = it->second["admin"];
                    auto result = admin.run_command(bsoncxx::builder::basic::make_document(bsoncxx::builder::basic::kvp("isMaster", 1)));
                    return true;
                }
            }

            const auto uri_str = settings.uri();
            mongocxx::uri uri(uri_str);
            auto client = mongocxx::client(uri);
            auto admin = client["admin"];
            auto result = admin.run_command(bsoncxx::builder::basic::make_document(bsoncxx::builder::basic::kvp("isMaster", 1)));
            return true;
        }
        catch (const std::exception& xcp)
        {
            return false;
        }
    }

    bool MongoDBConnectionManager::ConnectionExists(const MongoDBSettings& settings)
    {
        initialize_if();

        const auto uri_str = settings.uri();
        const auto& it = Connections.find(uri_str);
        return it != Connections.end();
    }
}
