#pragma once

#include <map>

#include <QTreeWidget>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/core/longtermmemory/Memory.h>

#include <RobotAPI/libraries/armem_gui/TreeWidgetBuilder.h>


namespace armarx::armem::gui::memory
{

    class TreeWidget : public QTreeWidget
    {
        Q_OBJECT
        using This = TreeWidget;

    public:

        TreeWidget();

        void update(const armem::wm::Memory& memory);
        void update(const std::map<std::string, const armem::wm::Memory*>& memories);

        std::optional<MemoryID> selectedID() const;


    signals:

        /// Tree contents have changed.
        void updated();

        /// The selected item or its contents have changed.
        void selectedItemChanged(const MemoryID& id);

        void itemSelected(const MemoryID& id);

        void memorySelected(const MemoryID& id);
        void coreSegmentSelected(const MemoryID& id);
        void providerSegmentSelected(const MemoryID& id);
        void entitySelected(const MemoryID& id);
        void snapshotSelected(const MemoryID& id);
        void instanceSelected(const MemoryID& id);


    private slots:

        void handleSelection();


    private:

        void initWidget();
        void initBuilders();

        void updateChildren(const armem::wm::Memory& memory, QTreeWidget* tree);
        void updateChildren(const std::map<std::string, const armem::wm::Memory*>& memories, QTreeWidget* tree);

        void updateChildren(const armem::wm::Memory& memory, QTreeWidgetItem* memoryItem);
        void updateChildren(const armem::wm::CoreSegment& coreSeg, QTreeWidgetItem* coreSegItem);
        void updateChildren(const armem::wm::ProviderSegment& provSeg, QTreeWidgetItem* provSegItem);
        void updateChildren(const armem::wm::Entity& entity, QTreeWidgetItem* entityItem);
        void updateChildren(const armem::wm::EntitySnapshot& snapshot, QTreeWidgetItem* snapshotItem);
        void updateChildren(const armem::wm::EntityInstance& data, QTreeWidgetItem* parent);


        QTreeWidgetItem* makeItem(const std::string& key, const armem::base::detail::MemoryItem& container);

        void updateItemItem(const armem::base::detail::MemoryItem& level, QTreeWidgetItem* item);
        template <class... T>
        void updateContainerItem(const armem::base::detail::MemoryContainerBase<T...>& container, QTreeWidgetItem* item);


    private:

        MapTreeWidgetBuilder<std::string, const wm::Memory*> workingmemoryBuilder;
        MapTreeWidgetBuilder<std::string, wm::CoreSegment> workingmemoryCoreSegmentBuilder;
        MapTreeWidgetBuilder<std::string, wm::ProviderSegment> workingmemoryProvSegmentBuilder;
        MapTreeWidgetBuilder<std::string, wm::Entity> workingmemoryEntityBuilder;
        MapTreeWidgetBuilder<armem::Time, wm::EntitySnapshot> workingmemorySnapshotBuilder;
        TreeWidgetBuilder<std::vector<wm::EntityInstance>> workingmemoryInstanceBuilder;

        std::optional<MemoryID> _selectedID;
        /// While this is false, do not handle selection updates.
        bool handleSelections = true;

        enum class Columns
        {
            KEY = 0,
            SIZE = 1,
            TYPE = 2,
            LEVEL = 3,
            ID = 4,
            NUM = 5
        };
    };

}

namespace armarx::armem::gui
{
    using MemoryTreeWidget = memory::TreeWidget;
}
