/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "List.h"

namespace armarx::aron::cppserializer::serializer
{
    // constructors
    ListSerializer::ListSerializer(const typenavigator::ListNavigatorPtr& e) :
        detail::ContainerSerializerBase<typenavigator::ListNavigator, ListSerializer>("std::vector<" + FromAronTypeNaviagtorPtr(e->getAcceptedType())->getFullCppTypename() + ">",
                simox::meta::get_type_name(typeid(data::AronList)),
                simox::meta::get_type_name(typeid(type::AronList)), e)
    {
    }

    CppBlockPtr ListSerializer::getResetSoftBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        std::string resolved_accessor = ResolveMaybeAccessor(accessor, typenavigator);

        block_if_data->addLine(accessor + nextEl() + "clear();");
        return ResolveMaybeResetSoftBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr ListSerializer::getResetHardBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine(accessor + " = " + this->getFullTypenameGenerator() + "();");
        return ResolveMaybeResetHardBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr ListSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());

        b->addLine("w.writeStartList({" + MAYBE_TO_STRING(typenavigator->getMaybe()) + "}); // of " + accessor);
        auto type_s = FromAronTypeNaviagtorPtr(typenavigator->getAcceptedType());

        CppBlockPtr b2 = type_s->getWriteTypeBlock(type_s->getFullCppTypename());
        b->appendBlock(b2);

        b->addLine("w.writeEndList(); // of " + accessor);
        return b;
    }

    CppBlockPtr ListSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("w.writeStartList(); // of " + accessor);

        std::string escaped_accessor = EscapeAccessor(accessor);
        std::string accessor_iterator = escaped_accessor + LIST_ITERATOR_ACCESSOR;

        auto type_s = FromAronTypeNaviagtorPtr(typenavigator->getAcceptedType());
        block_if_data->addLine("for(unsigned int " + accessor_iterator + " = 0; " + accessor_iterator + " < " + accessor + nextEl() + "size(); ++" + accessor_iterator + ")");
        block_if_data->addBlock(type_s->getWriteBlock(accessor + nextEl() + "at(" + accessor_iterator + ")"));
        block_if_data->addLine("w.writeEndList(); // of " + accessor);

        return ResolveMaybeWriteBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr ListSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());

        std::string escaped_accessor = EscapeAccessor(accessor);
        std::string accessor_iterator = escaped_accessor + LIST_ITERATOR_ACCESSOR;

        block_if_data->addLine("while(!r.readEndList()) // of " + accessor);
        CppBlockPtr b2 = CppBlockPtr(new CppBlock());

        auto type_s = FromAronTypeNaviagtorPtr(typenavigator->getAcceptedType());
        b2->addLine(type_s->getFullCppTypename() + " " + accessor_iterator + ";");
        b2->appendBlock(type_s->getReadBlock(accessor_iterator));
        b2->addLine(accessor + nextEl() + "push_back(" + accessor_iterator + ");");

        block_if_data->addBlock(b2);
        return ResolveMaybeReadBlock(accessor, "r.readStartList()", block_if_data, typenavigator);
    }

    CppBlockPtr ListSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
        block_if_data->addLine("\t return false;");
        return ResolveMaybeEqualsBlock(accessor, otherInstanceAccessor, block_if_data, typenavigator);
    }
}
