/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Orientation.h"

namespace armarx::aron::cppserializer::serializer
{
    // constructors
    OrientationSerializer::OrientationSerializer(const typenavigator::OrientationNavigatorPtr& n) :
        detail::NDArraySerializerBase<typenavigator::OrientationNavigator, OrientationSerializer>("Eigen::Quaternion<" + n->ACCEPTED_TYPE + ">",
                simox::meta::get_type_name(typeid(data::AronNDArray)),
                simox::meta::get_type_name(typeid(type::AronOrientation)), n)
    {
        ARMARX_CHECK_NOT_NULL(typenavigator);
    }

    CppBlockPtr OrientationSerializer::getResetHardBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine(accessor + " = " + this->getFullTypenameGenerator() + "();");
        return ResolveMaybeResetHardBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr OrientationSerializer::getResetSoftBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine(accessor + " = " + this->getFullCppTypename() + "();");
        return ResolveMaybeResetSoftBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr OrientationSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeOrientation({" + MAYBE_TO_STRING(typenavigator->getMaybe()) + "}); // of " + accessor);
        return b;
    }

    CppBlockPtr OrientationSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("w.writeNDArray({" + simox::alg::to_string(typenavigator->ACCEPTED_DIMENSION, ", ") + ", 4}, \"" + typenavigator->ACCEPTED_TYPE + "\", reinterpret_cast<const unsigned char*>(" + accessor + nextEl() + "coeffs().data())); // of " + accessor);

        return ResolveMaybeWriteBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr OrientationSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("r.readEndNDArray(reinterpret_cast<unsigned char*>(" + accessor + nextEl() + "coeffs().data())); // of " + accessor);
        return ResolveMaybeReadBlock(accessor, "r.readStartNDArray()", block_if_data, typenavigator);
    }

    CppBlockPtr OrientationSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("if (not (" + accessor + nextEl() + "isApprox(" + otherInstanceAccessor + ")))");
        block_if_data->addLine("\t return false;");
        return ResolveMaybeEqualsBlock(accessor, otherInstanceAccessor, block_if_data, typenavigator);
    }
}

