/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include "../detail/NDArraySerializerBase.h"

// ArmarX
#include "../../../../../navigator/type/ndarray/Orientation.h"

namespace armarx::aron::cppserializer::serializer
{
    class OrientationSerializer;
    typedef std::shared_ptr<OrientationSerializer> AronOrientationTypeCppSerializerPtr;

    class OrientationSerializer :
            virtual public detail::NDArraySerializerBase<typenavigator::OrientationNavigator, OrientationSerializer>
    {
    public:
        using PointerType = AronOrientationTypeCppSerializerPtr;

    public:
        // constructors
        OrientationSerializer(const typenavigator::OrientationNavigatorPtr&);

        // virtual implementations
        virtual CppBlockPtr getResetHardBlock(const std::string& accessor) const override;
        virtual CppBlockPtr getResetSoftBlock(const std::string& accessor) const override;
        virtual CppBlockPtr getWriteTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteBlock(const std::string&) const override;
        virtual CppBlockPtr getReadBlock(const std::string&) const override;
        virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;

    private:
        static constexpr const char* DIMENSION_ACCESSOR = "_dimensions";
        static constexpr const char* TYPE_ACCESSOR = "_type";
    };
}
