/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "List.h"

namespace armarx::aron::typenavigator
{
    // constructors
    ListNavigator::ListNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eList, path)
    {
    }

    ListNavigator::ListNavigator(const type::AronListPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eList, path),
        detail::NavigatorBase<type::AronList, ListNavigator>(o),
        acceptedType(FromAronType(o->acceptedType))
    {
    }

    // operators
    bool ListNavigator::operator==(const ListNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }
        return getAcceptedType() == other.getAcceptedType();
    }

    // Member functions
    NavigatorPtr ListNavigator::getAcceptedType() const
    {
        return acceptedType;
    }

    void ListNavigator::setAcceptedType(const NavigatorPtr& a)
    {
        ARMARX_CHECK_NOT_NULL(a);
        this->aron->acceptedType = a->toAronTypePtr();
        acceptedType = a;
    }

    // static methods
    type::AronListPtr ListNavigator::toAronListPtr() const
    {
        return this->aron;
    }

    // virtual implementations
    std::vector<NavigatorPtr> ListNavigator::getChildren() const
    {
        return {acceptedType};
    }

    size_t ListNavigator::childrenSize() const
    {
        return 1;
    }

    std::string ListNavigator::getName() const
    {
        return "AronListType<" + acceptedType->getName() + ">";
    }
}
